function [Pires,Sigmares,UpsRes,Phires,MeanUpsilon,StdUpsilon,preds,LogPost,NRestrict,MA,HistDecompPreds,EStructCollect]=GibbsBVAR(X,D,K,...
    niter,NBurnin,ThinOutFreq,difference,exogenous,lambdas,firstlagmean,LoadPriorFromFile,MeanAdj,muphi,Sigmaphi,domesticdummies,FutureD,horizon,...
    IncludeConstant,ML,ComputeLogPost,StructuralForm,Grest,g,WZNorm,DfCorrML,IRHorizon,VarLabels,PriorSeason,SeasonalVars,HistDecompShocks,...
    HistDecompStartObs,RandSeed,CondHorizon,CondVar,CondShock,CondPath,CondPathInAnnualGrowth,OtherShocks,FinalEstDate,ForceStationarity,...
    StoreStructShocks)
                                                                                                                                                            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Gibbs sampling from the posterior distribution of a VAR model. Standard or Steady state VAR. Structural or reduced form. 
%           Impulse respons analysis, historical decomposition, unconditional and conditional forecasting.
%
% MODEL:    Upsilon'*x_{t}=PI_{1}*x_{t-1}+...+PI_{K}*x_{t-K}+PHI*d_{t}+eps_{t},
%
%           where x_{t} is p-dimensional and d_{t} is q-dimensional.
%									
% INPUT:	X: 		        (T-by-p)            Matrix with T observations on p endogenous variables.
%			D: 		        (T-by-q)            Matrix with T observations on q exogenous variables. 
%			K:              (scalar)            Number of lags in the VAR.
%           niter:          (scalar)            Number of post-burnin Gibbs iterations
%           NBurnin         (Scalar)            Number of burn-in posterior draws.
%           ThinOutFreq     (Scalar)            Thin-out factor for draws of ir, hist. decomp. and structural shocks.
%                                               If ThinOutFreq=10, then every tenth draw of the IRs etc are saved.
%           difference      (NDiffVars-vector)  Vector with indicies for the endogenous Difference variables.     
%           exogenous       (NExoVars-vector)   Indicies for the columns in X (variables) which are exogenous with 
%                                               respect to the other variables in the model.
%           lambdas         (6-vector)          Hyperparameters in prior
%                                               Smaller values produces priors which are more concentrated around zero.
%                                               - lambdas(1): Baseline tightness parameter.         Limits=(0,inf)  
%                                               - lambdas(2): Own/others shrinkage                  Limits=(0,1)    
%                                               - lambdas(3): Lag length shrinkage parameter.       Limits=(0,inf)  
%                                               - lambdas(4): Exogenity parameter                   Limits=(0,1)      
%                                               - lambdas(5): Determ. trends parameter              Limits=(0,inf)  
%                                               - lambdas(6): Seasonal shrinkage                    Limits=(0,1].
%           firstlagmean    (1-by-p)            Mean of first first own lag coefficient. If empty firstlagmean contains zeros or ones as
%                                               determined by difference (a variable in levels has unity mean and a variable is differences has zero mean).
%           LoadPriorFromFile (0-1)             If =1, prior is loaded from file ManualPrior.mat
%           MeanAdj         (0-1)               Determines the way the constant is handled. If MeanAdj==1, mean-adjusted/steady state form is used:
%                                               PI_{0}*(x_{t}-PSI*d_{t})=PI_{1}*(x_{t-1}-PSI*d_{t-1})+...+eps_{t},
%           muphi           (q-by-1)            Prior mean of deterministic coefficients in the Steady State VAR (Psi).
%           Sigmaphi        (q-by-q)            Prior Covariance matrix of deterministic coefficients in the Steady State VAR (Psi).
%           domesticdummies (1-by-q)            Vector indicating for each deterministic variable if it is should affect only the domestic variables.
%           FutureD         (horizon-by-q)      Matrix with values on the deterministic variables during the forecasting period. If
%                                               empty, the last row of D is replicated horizon times.
%           horizon         (scalar)            Forecasting horizon
%           IncludeConstant (0-1)               =1, constant is included in the estimation.
%           ML              (0-1)               =1, Maximum likelihood  analysis, otherwise Bayesian analysis. Note currently ML is
%                                               without standard errors and some errors will occur. Ignore them.
%           ComputeLogPost  (0-1)               =1, Log posterior is compute for each posterior draw. 
%           StructuralForm  (scalar)            =0 - Reduced form
%                                               =1 - Recursive identified VAR - Choleski lower triangular PI_{0}.
%                                               =2 - Non-recursive VAR is estimated with restrictions given by Grest.
%           Grest           (NVars-by-NVars)    Matrix with restrictions on the contemporaneous coefficients. Grest(i,j)=1 means that the
%                                               coefficient on variable j is unrestricted in the i:th equation. Grest(i,j)=0 forces it to be zero.
%           g               (NVars-by-1)        Normalizing variables. g(i)=k means that in the k:th unrestricted coefficient in the
%                                               i:th equation is normalized to be positive.
%           WZNorm          (0-1)               =1 - Waggoner-Zha (JoE,2003) normalization of the contemporaneous coefficients
%           DfCorrML        (0-1)               =1, ML estimate of PI_{0} is based on degrees-of-freedom corrected estimate of the error covariance matrix.
%           IRHorizon       (scalar)            The number of time period where the impulse response function is computed.
%           VarLabels       (NVars-by-1 cell)   Cell array with variable labels.
%           PriorSeason     (Scalar)            Season (1=yearly, 4=quarterly, 12=monthly)
%           SeasonalVars    (NSeasonal-by-1)    Seasonal variables.
%           HistDecompShocks (Cell)             Sets of constructive shocks in the conditional forecasts.
%           HistDecompStartObs (String)         Date where the shocks in the historical decompositions are turned off.
%           RandSeed        (String or scalar)  If 'random', then seed to random number generator is set by the CPU clock. Else seed=RandSeed.
%           CondHorizon     (Cell)              Horizons where the conditions on the endogenous variable are imposed.
%           CondVar         (Cell)              Indicies of the blocks of variables that are restricted in the conditional forecasts.
%           CondShock       (Cell)              Indicies of the blocks of constructive shocks in the conditional forecasts.
%           CondPath        (Cell)              Paths of the restricted endogenous variables.
%           CondPathInAnnualGrowth (0-1)        If =1, then CondPath are specified in annual growth rates for Difference variables
%           OtherShocks     (0-1)               If =1, the non-constructive shocks are drawn from their distribution during the forecast stage.
%           FinalEstDate    (String)            Date for the end of the estimation sample.
%           ForceStationarity (0-1)                 If =1, VAR is restricted to be stationary. 
%           StoreStructShocks (0-1)             If =1, then structural shocks are stored, and subsequently plotted.
%
% OUTPUT:	Pires           (niter-by-(p^2)*k+p*q)  Matrix of posterior draws of VAR coefficients and coefficients on exogenous variables.
%           Sigmares        (niter-by-p*(p-1)/2)    Matrix of posterior draws of the covariance matrix of disturbances.
%           UpsRes          (niter-by-s)            Matrix of posterior draws of the contemporaneous coefficients
%           Phires          (niter-by-p*q)          Matrix of posterior draws of the coefficients on non-modelled variables (only in MeanAdj mode)
%           MeanUpsilon     (p-by-p)                Posterior mean of Upsilon
%           StdUpsilon      (p-by-p)                Posterior standard deviation of Upsilon
%           preds           (niter-by-NVars-horizon)3-dim matrix with draws from predictive distribution.
%           LogPost         (niter-by-1)            Posterior density at each draw.
%           NRestrict       (scalar)                The number of restricted parameters in the ML estimate
%           MA              (NVars-NVars-niterThin) Posterior draws of MA (=IR) coefficients
%           HistDecompPreds (niterThin,NVars,T,NDecomp) Posterior draws of the historical decompositions.
%           EStructCollect  (T-by-NVars-niterThin)  Posterior draws of the structural shocks.
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@riksbank.se.
%
% FIRST VER: 2003-10-10
% REVISED:	 2007-09-29
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%% User input %%%%%%%%%%%%%
NonStationaryLetGo=10; % If 10 failures to generate stationary Pi's, then move on to next iteration.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Set seed to random number generator.
if RandSeed==0
    rand('state',sum(100*clock))
    randn('state',sum(100*clock))
else
    rand('state',RandSeed)
    randn('state',RandSeed)
end  

% Prelims - Historical Decompositions
if ML>0
    ThinOutFreq=1;
end
ThinOutCount=0; % Counter used to subsample the posterior draws of the historical decompositions.

if isnan(HistDecompShocks{1}) % Don't do the decomposition
    ComputeHistDecompShocks=0; 
else
    ComputeHistDecompShocks=1;
end
if HistDecompStartObs==0
    TShutDown=1;
else
    TShutDown=HistDecompStartObs;
end

NRestrict=nan;

if ML>0
    niter=2;
end

if MeanAdj==1
    muphi=muphi(:);
    Sigmaphi=Sigmaphi(:);    
end

% Adding a constant to D
T=size(X,1);
if IncludeConstant>0 
    D=[D ones(T,1)];
end

q=size(D,2);

% Construct future exogenous
if q>0 && size(FutureD,1)==0
    FutureD=repmat(D(T,:),horizon,1);
elseif q==0
    FutureD=zeros(horizon,1);
end

Data=X;
Doriginal=D;
% Transform data to multivariate regression form: Y=XB+E.
[Y,X,D]=transform_to_multireg(X,D,K);
X=[X D];
[T,p]=size(Y);

% Prelims - Conditional forecasts
NCondBlocks=length(CondVar); % Number of blocks of conditioning variables in the conditional forecasts.
CondHorizonAll=[];
for j=1:NCondBlocks
    CondHorizonAll=[CondHorizonAll;CondHorizon{j}];
    % Converting the annual growth CondPath to quarterly growth
    if CondPathInAnnualGrowth && any(ismember(CondVar{j},difference))  % Checks if CondVar is among the converted variables.
        LastData=Y(end-2:end,CondVar{j});
        for var=1:length(CondVar{j})
            if ismember(CondVar{j}(var),difference)
                for h=1:length(CondHorizon{j})
                    CondPath{j}(h,var)=CondPath{j}(h,var)-sum(LastData(:,var));
                    LastData(:,var)=[LastData(2:3,var);CondPath{j}(h,var)];
                end
            end
        end
    end
end
CondHorizonAll=sort(unique(CondHorizonAll)); % This is a vector with all horizons where there is at least one conditioning.

%%%%%%%%%%%%%%%%%% Construct Small open economy modified Minnesota prior %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:p
    scales(i)=univariatevar(Y(:,i),K,D);
end
if LoadPriorFromFile==1
    load('ManualPrior') % This file must contain mu_pi and Omega_pi if Steady State VAR, otherwise must contain mu and Omega.
else
    if MeanAdj==1
        [mu_pi,Omega_pi]=build_soe_minn(p,exogenous,q,K,scales,lambdas,difference,firstlagmean,domesticdummies,IncludeConstant,PriorSeason,SeasonalVars); % mu and Omega is for vec(B).
        % Prior on Phi
        mu_phi=muphi;
        Omega_phi=diag(Sigmaphi.^2);
    else
        [mu,Omega]=build_soe_minn(p,exogenous,q,K,scales,lambdas,difference,firstlagmean,domesticdummies,IncludeConstant,PriorSeason,SeasonalVars); % mu and Omega is for vec(B).
    end
end

%%%%%%%%%%%%%      Handling the restrictions, initializing result matrices etc.    %%%%%%%%%%%%%%%%%%%%%%%%%
switch StructuralForm
    case 0 % Reduced form - No impulse response analysis
        fprintf('\n')
        Sigmares=zeros(niter,p*(p+1)/2);
        UpsRes=nan;
        MA=nan;
    case 1 % Recursive/Choleski
        fprintf('\n')
        UpsRes=zeros(niter,p*(p+1)/2);
        if IRHorizon>0
            MA=zeros(p,p,IRHorizon+1,floor(niter/ThinOutFreq));
        else
            MA=nan;
        end
        Sigmares=zeros(niter,p*(p+1)/2);
    case 2 % Non-recursive
        fprintf('\n')
        % Handling the restrictions on the contemporaneous coefficients
        if WZNorm==1
            G=makeG(Grest);
            UnitVectMatrix=eye(p,p);
        elseif isempty(g) % Default normalization of variables ith coefficient in equation i.
            [G,g]=makeG(Grest);
        else % Use g to normalize
            G=makeG(Grest);
        end
        
        % Defines the number of free parameters in each simultaneous equation vector.
        si=zeros(p,1);
        for i=1:p
            si(i)=size(G{i},2);
        end
        s=sum(si);
        delimit=[0;cumsum(si)];
        
        UpsRes=zeros(niter,s);
        Sigmares=nan;
        if IRHorizon>0
            MA=zeros(p,p,IRHorizon+1,floor(niter/ThinOutFreq));
        else
            MA=nan;
        end
    otherwise
end
UpsilonCum=zeros(p,p);
UpsilonCumSquare=zeros(p,p);

if MeanAdj==1
    Phires=zeros(niter,p*q);
    Pires=zeros(niter,p^2*K);
else
    Pires=zeros(niter,p^2*K+p*q);
    Phires=[];
end
preds=zeros(niter,p,horizon);
if ComputeHistDecompShocks==1
    HistDecompPreds=zeros(floor(niter/ThinOutFreq),p,T,length(HistDecompShocks));
else
    HistDecompPreds=nan;
end

if StoreStructShocks
    EStructCollect=zeros(T,p,floor(niter/ThinOutFreq));
else
    EStructCollect=nan;
end

if ComputeLogPost==1
    LogPost=zeros(niter,1);
else 
    LogPost=nan;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%% Computing ML estimates %%%%%%%%%%%%%%%%%%%
XX=X'*X;
invXX=inv(XX);
switch StructuralForm
    case 0 % Reduced form
        if MeanAdj==1
            [Pihat,Phihat,Sigmahat]=mlvarmeanadj(Data,Doriginal,K);
        else
            Phihat=[];
            if ML>1 % Block restiction of exogenous variables
                if ML==2
                    QuasiML=0; % FIML
                else
                    QuasiML=1; % Quasi ML
                end
                [Pihat,Sigmahat,NRestrict]=MLRestrictVAR(Data,Doriginal,K,[],exogenous,QuasiML);
            else % 
                Pihat=invXX*X'*Y;
            end
            if DfCorrML==1
                Sigmahat=(Y-X*Pihat)'*(Y-X*Pihat)/(T-K*p);
            else
                Sigmahat=(Y-X*Pihat)'*(Y-X*Pihat)/T;
            end
        end
    case 1 % Recursive/Choleski
        if MeanAdj==1
            [Pihat,Phihat,Sigmahat]=mlvarmeanadj(Data,Doriginal,K);
        else
            Phihat=[];
            if ML>1 % Block restriction of exogenous variables
                if ML==2
                    QuasiML=0; % FIML
                else
                    QuasiML=1; % Quasi ML
                end
                [Pihat,Sigmahat,NRestrict]=MLRestrictVAR(Data,Doriginal,K,[],exogenous,QuasiML);
            else
                Pihat=invXX*X'*Y; % Unrestricted model
            end
            if DfCorrML==1
                Sigmahat=(Y-X*Pihat)'*(Y-X*Pihat)/(T-K*p);
            else
                Sigmahat=(Y-X*Pihat)'*(Y-X*Pihat)/T;
            end
        end
        
        Upsilon=inv(chol(Sigmahat));
        % Pihat=PihatRF*Upsilon; % Pihat is here on structural form
    case 2 % Non-recursive
        [UpsilonHat,Pihat]=MLEstStructuralVAR(Data,Doriginal,K,Grest,g,0,DfCorrML);
        if MeanAdj==1 % This gives us an approximation of the ML (it is exact if we use exactly identifying restrictions),
            [Pihat,Phihat,Sigmahat]=mlvarmeanadj(Data,Doriginal,K);  % but its OK for starting values of the Gibbs.
        else
            Phihat=[];
        end
    otherwise
end % End switch StructuralForm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%% Initial values %%%%%%%%%%%%%%%%%%%%%%%%%%
switch StructuralForm
    case 0 % Reduced form
        Pi=Pihat;   % MLE used as Initial values
    case 1 % Recursive/Choleski
        Pi=Pihat;   % MLE used as Initial values
    case 2 % Non-recursive
        Upsilon=UpsilonHat;
        InvUpsilon=inv(Upsilon);
        Pi=Pihat;   % MLE used as Initial values
    otherwise
end
vecPiOld=vec(Pi);

Phi=Phihat;

% Computing quantities in the full conditional posteriors that do not vary during the iterations.
if MeanAdj==1
    % Computes datamatrices to be used in the full conditional posterior of Phi
    [Yma,Xma,Dma]=datmanip_meanadj(Data,Doriginal,K);
    [T,p]=size(Yma);
    DD=Dma'*Dma;
    InvOmega_pi=inv(Omega_pi);
    InvOmega_pi_mu=InvOmega_pi*mu_pi;
    InvOmega_phi=inv(Omega_phi);
    invOmega_phi_mu=InvOmega_phi*mu_phi;
    U=[eye(p*q) zeros(p*q,p*q*K)]';
else
    invOmega=diag(1./diag(Omega));
    invOmega_mu=invOmega*mu;
    XX_Pihat=XX*Pihat;
end

switch StructuralForm
    case 0
        if MeanAdj==1
            ModelType='Steady State VAR - Reduced Form';
        else
            ModelType='Standard VAR - Reduced Form';
        end
    case 1
        if MeanAdj==1
            ModelType='Steady State VAR - Recursive Identification';
        else
            ModelType='Standard VAR - Recursive Identification';
        end
    case 2
        if MeanAdj==1
            ModelType='Steady State VAR - General Identification';
        else
            ModelType='Standard VAR - General Identification';
        end
    otherwise
end

MHStartTime=clock;
WaitBarHandle=waitbar(0,strcat(['Time left:  ',' ','',' ','']),'name',[ModelType,' ',FinalEstDate]);
set(WaitBarHandle,'resize','on')

% Define the matrices which appear in the steady state form. Updated every Gibbs iteration.
if MeanAdj==1
    [Yphi,Xphi]=datmanip_cond_on_phi(Data,Doriginal,Phi,K);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% BEGIN GIBBS SAMPLING %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=1:niter
    
    if rem(i,ThinOutFreq)==0 % Some things (IR, Structural shocks etc) may be stored only at certain iterations.
        ThinOutCount=ThinOutCount+1;
    end

    % Display WaitBar
    if rem(i,100)==0
        TotalSecLeft=etime(clock,MHStartTime)*((niter-i)/i);
        HoursLeft=floor(TotalSecLeft/3600);
        MinLeft=floor((TotalSecLeft-HoursLeft*3600)/60);
        SecLeft=floor(TotalSecLeft-HoursLeft*3600-MinLeft*60);
        waitbar(i/niter,WaitBarHandle,strcat(['Time left: ',sprintf('%2.0f',HoursLeft),' h ',sprintf('%2.0f',MinLeft),' min ',sprintf('%2.0f',SecLeft),' sec ']),'name',ModelType);
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%    Sigma/Upsilon step    %%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    switch StructuralForm

        case 0 % Reduced form
            if ML>0
                Sigma=Sigmahat;
                E=Y-X*Pihat;
            else
                if MeanAdj==1
                    E=Yphi-Xphi*Pi;
                else
                    E=Y-X*Pi;
                end
                Sigma=0;
                while det(Sigma)==0 || isnan(det(Sigma))
                    Sigma=randiw(E'*E,T);
                end
                Sigmares(i,:)=vech(Sigma)';
            end
            invSigma=inv(Sigma);
            InvUpsilon=chol(Sigma);
            Upsilon=inv(InvUpsilon);
        case 1 % Recursive/Choleski
            if ML>0
                Sigma=Sigmahat;
                E=Y-X*Pihat;
            else
                if MeanAdj==1
                    E=Yphi-Xphi*Pi;
                else
                    E=Y-X*Pi;
                end
                Sigma=0;
                while det(Sigma)==0 | isnan(det(Sigma))
                    Sigma=randiw(E'*E,T);
                end
            end
            
            Sigmares(i,:)=vech(Sigma)';
            invSigma=inv(Sigma);
            InvUpsilon=chol(Sigma);
            Upsilon=inv(InvUpsilon);
            UpsRes(i,:)=vech(Upsilon')';

        case 2 % Non-recursive

            if ML==0
                if MeanAdj==1
                    UpsHat=inv(Yphi'*Yphi)*Yphi'*Xphi*Pi;
                else
                    UpsHat=inv(Y'*Y)*Y'*X*Pi;
                end
                for j=1:p
                    if MeanAdj==1
                        Cbar=T*inv(G{j}'*Yphi'*Yphi*G{j});
                        cbar=(1/T)*Cbar*(G{j}'*Yphi'*Yphi*UpsHat(:,j));
                    else
                        Cbar=T*inv(G{j}'*Y'*Y*G{j});
                        cbar=(1/T)*Cbar*(G{j}'*Y'*Y*UpsHat(:,j));
                    end
                    Q=chol(Cbar)';
                    delcol(Upsilon,j)';
                    UpsOrth=null(delcol(Upsilon,j)');
                    w1=Q'*G{j}'*UpsOrth;
                    w1=w1/norm(w1);
                    W=[w1 null(w1')];

                    betahat=(cbar'*inv(Q')*W)';
                    clear beta
                    beta(1)=absolutenormal(betahat(1),T);
                    if length(betahat)>1
                        beta(2:si(j))=betahat(2:si(j))+randn(si(j)-1,1)*sqrt(1/T);
                    end
                    psitemp=sum((W*diag(beta))')*Q';

                    % Normalization
                    if WZNorm==1 % Waggoner-Zha (JoE, 2003) normalization
                        Upsilon(:,j)=G{j}*psitemp';
                        InvUpsilon=inv(Upsilon);
                        if UnitVectMatrix(:,j)'*InvUpsilon*UpsilonHat(:,j)<0
                            psitemp=-psitemp;
                            Upsilon(:,j)=-Upsilon(:,j);
                            InvUpsilon=inv(Upsilon);
                        end
                    else % normalize by g-vector
                        if psitemp(g(j))<0
                            psitemp=-psitemp;
                        end
                        Upsilon(:,j)=G{j}*psitemp';
                        InvUpsilon=inv(Upsilon);
                    end
                    UpsRes(i,1+delimit(j):delimit(j+1))=psitemp;
                end
            end
            invSigma=eye(p);
        otherwise
    end
    
    if i>NBurnin
        UpsilonCum=UpsilonCum+Upsilon;
        UpsilonCumSquare=UpsilonCumSquare+Upsilon.^2;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%     PI-step   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    IsStationary=0;
    NStationaryTrials=0;
    while ~IsStationary % Keep on drawing until VAR is stationary.
        if ML>0
            vecPi=vec(Pi);
        else
            if MeanAdj==1 % Mean adjusted form
                Omega_pi_bar=inv(kron(invSigma,Xphi'*Xphi)+InvOmega_pi);
                mu_pi_bar=Omega_pi_bar*(vec(Xphi'*Yphi*invSigma)+InvOmega_pi_mu);
                vecPi=randmn(mu_pi_bar,Omega_pi_bar); % vecPi=vec([Pi_{1},...,Pi{K}]')
                Pi=invvec(vecPi,p*K); % Pi=[Pi_{1},...,Pi{K}]';
                Piprime=Pi'; % Piprime=[Pi_{1},...,Pi{K}];
                Pires(i,:)=vecPi';
            else % Standard form
                % Note that Pi is in reduced form here if recursive identifying restriction option is used
                Omegatilde=inv(kron(invSigma,XX)+invOmega); %Pi=[Pi_{1},...,Pi_{K} THETA]'
                mutilde=Omegatilde*(vec(XX_Pihat*invSigma)+invOmega_mu);
                vecPi=randmn(mutilde,Omegatilde);
                Pi=invvec(vecPi,p*K+q); % Pi=[Pi_{1},...,Pi_{K} THETA]'
            end
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%   Computing reduced form matrices  %%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        switch StructuralForm
            case {0,1} % Reduced Form or Recursive Structural Form
                PiReduced=Pi;
                PiReducedT=PiReduced'; % PiReducedT=[A_{1} A_{2},...,A_{K} Phi] in the reduced non-condensed form
                % x_{t}=A_{1}*x_{t-1}+...,A_{K}*x_{t-K}+Phi*d_{t}+eps
            case 2 % Non-recursive Structural Form
                PiReduced=Pi*InvUpsilon;
                PiReducedT=PiReduced'; % PiReducedT=[A_{1} A_{2},...,A_{K} Phi] in the reduced non-condensed form
                % x_{t}=A_{1}*x_{t-1}+...,A_{K}*x_{t-K}+Phi*d_{t}+eps
            otherwise
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % Check for stationarity
        % PiReducedT(:,1:p*K) is [A_{1} A_{2},...,A_{K}] in the reduced non-condensed form x_{t}=A_{1}*x_{t-1}+...,A_{K}*x_{t-K}+eps
        if ForceStationarity && NStationaryTrials<10
            NStationaryTrials=NStationaryTrials+1;
            CompanionPi =  [PiReducedT(:,1:p*K) ;[eye(p*(K-1)),zeros(p*(K-1),p)]];
            if any(abs(eig(CompanionPi))>=1)
                % % Non-stationary. IsStationary is still zero. Keep looping to find stationaty draw.
            else
                IsStationary=1;
            end
        else
            IsStationary=1;
        end
        
    end
    if NStationaryTrials==NonStationaryLetGo && ~IsStationary % Failed to find a stationary Pi
        vecPi=vecPiOld;
        Pires(i,:)=vecPi';
        if MeanAdj==1 % Mean adjusted form
            Pi=invvec(vecPi,p*K); % Pi=[Pi_{1},...,Pi{K}]';
        else
            Pi=invvec(vecPi,p*K+q); % % Pi=[Pi_{1},...,Pi_{K} THETA]'
        end
        switch StructuralForm
            case {0,1} % Reduced Form or Recursive Structural Form
                PiReduced=Pi;
                PiReducedT=PiReduced'; % PiReducedT=[A_{1} A_{2},...,A_{K} Phi] in the reduced non-condensed form
                % x_{t}=A_{1}*x_{t-1}+...,A_{K}*x_{t-K}+Phi*d_{t}+eps
            case 2 % Non-recursive Structural Form
                PiReduced=Pi*InvUpsilon;
                PiReducedT=PiReduced'; % PiReducedT=[A_{1} A_{2},...,A_{K} Phi] in the reduced non-condensed form
                % x_{t}=A_{1}*x_{t-1}+...,A_{K}*x_{t-K}+Phi*d_{t}+eps
            otherwise
        end
    else % Stationary Pi found
        Pires(i,:)=vecPi';
        vecPiOld=vecPi;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%   Phi step - only for mean adjusted form  %%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if MeanAdj==1
        % Phi-step
        for j=1:K
            U(1+j*p*q:(j+1)*p*q,:)=kron(eye(q),Piprime(:,1+(j-1)*p:j*p));
        end
        Omega_phi_bar=inv(U'*kron(DD,invSigma)*U+InvOmega_phi);
        mu_phi_bar=Omega_phi_bar*(U'*vec(invSigma*(Yma-Xma*Pi)'*Dma)+invOmega_phi_mu);
        vecPhi=randmn(mu_phi_bar,Omega_phi_bar); % vecPhi=vec(Phi)
        Phi=invvec(vecPhi,p);
        Phires(i,:)=vecPhi';
        [Yphi,Xphi]=datmanip_cond_on_phi(Data,Doriginal,Phi,K);
    end
        
    % Computing Reduced Form and Structual Form Residuals
    if MeanAdj==1
        E=Yphi-Xphi*PiReduced;
    else
        E=Y-X*PiReduced;
    end
    EStruct=E*Upsilon;
    
    if StoreStructShocks && rem(i,ThinOutFreq)==0
        EStructCollect(:,:,ThinOutCount)=EStruct;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%% Impulse response functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if IRHorizon>0 && StructuralForm>0 && rem(i,ThinOutFreq)==0
        PiReducedTNoPhi=PiReducedT(:,1:p*K); % PiReducedTNoPhi=[A_{1} A_{2},...,A_{K}] in the reduced non-condensed form x_{t}=A_{1}*x_{t-1}+...,A_{K}*x_{t-K}+eps
        MA(:,:,1,ThinOutCount)=InvUpsilon'; % InvUpsilon'=inv(A_{0})
        for h=2:IRHorizon+1
            for w=1:h-1
                if w<=K
                    MA(:,:,h,ThinOutCount)=MA(:,:,h,ThinOutCount)+PiReducedTNoPhi(:,1+(w-1)*p:w*p)*MA(:,:,h-w,ThinOutCount);
                end
            end
        end
    end    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    %%%%%%%%%%%%%%%%%%%%%%%%%     Prediction step      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    
    % Define the recent history of the process. Used when forming the predictions. Reset in every Gibbs iteration 
    if MeanAdj==1 
        Recent=[Yphi(T,:) Xphi(T,1:p*(K-1))]; 
    else 
        Recent=[Y(T,:) X(T,1:p*(K-1))]; 
    end

    InvUpsilonT=inv(Upsilon');
    CondHorizonCount=zeros(NCondBlocks,1); 
    for h=1:horizon
        % Defining the shock sequence 
        if ~ismember(h,CondHorizonAll) && ML==0 % No variable is restricted at horizon h and we do Bayesian estimation
            Shock=randn(1,p)*InvUpsilon; 
        else   % At least one variable is restricted at horizon h 
            Shock=zeros(1,p); 
            
            % Finding out which shocks are not used to produce a conditioning at horizon h 
            if OtherShocks==1 && ML==0 % The passive shocks are drawn from their distribution. 
                CondShockAll=[]; 
                for j=1:NCondBlocks 
                    CondShockAll=[CondShockAll CondShock{j}]; 
                end 
                NoCondShock=setdiff(1:p,unique(CondShockAll)); 
                Shock(NoCondShock)=randn(1,length(NoCondShock));  
            end 
            for j=1:NCondBlocks 
                if any(CondHorizon{j}==h)==1 && ML==0 % Checks if the current conditioning-block has restrictions at the current horizon

                    CondHorizonCount(j)=CondHorizonCount(j)+1; 
                    % Computes unconditional forecasts 
                    if MeanAdj==1 
                        UnCondPred=FutureD(h,:)*Phi'+Recent*PiReduced+Shock*InvUpsilon; 
                    else 
                        UnCondPred=[Recent FutureD(h,:)]*PiReduced+Shock*InvUpsilon;
                    end 
                    % Checks if some variables within the current conditioning block is zero at the current conditioning horizon. If so, then remove.
                    CurrentCondPath=CondPath{j}(CondHorizonCount(j),:); 
                    ActiveCondVar=isnan(CurrentCondPath)==0; 
                    CurrentCondVar=CondVar{j}(ActiveCondVar); 
                    CurrentCondPath=CurrentCondPath(ActiveCondVar); 
                    Shock(CondShock{j})=(CurrentCondPath-UnCondPred(CurrentCondVar))*inv(InvUpsilonT(CurrentCondVar,CondShock{j}))';
                end 
            end 
            Shock=Shock*InvUpsilon; % Going from structural shocks to reduced form shocks    
        end % End definition of shock sequences
        
        % Producing the forecasts 
        if MeanAdj==1 % Mean adjusted form 
            preds(i,:,h)=FutureD(h,:)*Phi'+Recent*PiReduced+Shock; 
            Recent=[preds(i,:,h)-FutureD(h,:)*Phi' Recent(1:p*(K-1))]; 
        else % Standard form 
            preds(i,:,h)=[Recent FutureD(h,:)]*PiReduced+Shock; 
            Recent=[preds(i,:,h) Recent(1:p*(K-1))]; 
        end 
    end % end h - loop over forecast horizon.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%     Historical decomposition step      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if ComputeHistDecompShocks==1 && rem(i,ThinOutFreq)==0

        for DecompCounter=1:length(HistDecompShocks)
            StructShock=zeros(size(EStruct));
            % Define initial values for baseline prediction
            if MeanAdj==1
                RecentFirst=[Xphi(1,1:p*K)];
            else
                RecentFirst=[X(1,1:p*K)];
            end
            
            % Predicting from the beginning of the sample, injecting the requested estimated structural shocks.
            for h=1:T
                StructShock=zeros(1,p);
                if h<TShutDown % Shocks are still active (not shut down) at time t.
                    StructShock=EStruct(h,:);
                else % Some shocks are shut down, only fill up StructShock with the active shocks.
                    StructShock(HistDecompShocks{DecompCounter})=EStruct(h,HistDecompShocks{DecompCounter});
                end
                
                if MeanAdj==1 % Mean adjusted form
                    HistDecompPreds(ThinOutCount,:,h,DecompCounter)=D(h,:)*Phi'+RecentFirst*PiReduced+StructShock*InvUpsilon; % Y(2) is the first predicted obs.
                    RecentFirst=[HistDecompPreds(ThinOutCount,:,h,DecompCounter)-D(h,:)*Phi' RecentFirst(1:p*(K-1))];
                else % Standard form
                    HistDecompPreds(ThinOutCount,:,h,DecompCounter)=[RecentFirst D(h,:)]*PiReduced+StructShock*InvUpsilon; % Y(2) is the first predicted obs.
                    RecentFirst=[HistDecompPreds(ThinOutCount,:,h,DecompCounter) RecentFirst(1:p*(K-1))];
                end
            end % End forecast horizon loop
        end % End DeCompCounter
        
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%   Computing the log posterior/likelihood of the draw   %%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if ComputeLogPost==1
        if StructuralForm==2 % Non-recursive
            if MeanAdj==1
                if ML>0
                    LogPrior=0;
                else
                    LogPrior=-(1/2)*log(det(2*pi*Omega_pi))-(1/2)*(vecPi-mu_pi)'*InvOmega_pi*(vecPi-mu_pi)...
                        -(1/2)*log(det(2*pi*Omega_phi))-(1/2)*(vecPhi-mu_phi)'*InvOmega_phi*(vecPhi-mu_phi);
                end
                LogPost(i)=-((T*p)/2)*log(2*pi)+T*log(det(Upsilon))-(1/2)*trace((Yphi*Upsilon-Xphi*Pi)'*(Yphi*Upsilon-Xphi*Pi))+LogPrior;
            else
                if ML>0
                    LogPrior=0;
                else
                    LogPrior=-(1/2)*log(det(2*pi*Omega))-(1/2)*(vecPi-mu)'*invOmega*(vecPi-mu);
                end
                LogPost(i)=-((T*p)/2)*log(2*pi)+T*log(det(Upsilon))-(1/2)*trace((Y*Upsilon-X*Pi)'*(Y*Upsilon-X*Pi))+LogPrior;
            end
        else % Reduced form or recursive
            if MeanAdj==1
                if ML>0
                    LogPrior=0;
                else
                    LogPrior=-(1/2)*log(det(2*pi*Omega_pi))-(1/2)*(vecPi-mu_pi)'*InvOmega_pi*(vecPi-mu_pi)...
                        -(1/2)*log(det(2*pi*Omega_phi))-(1/2)*(vecPhi-mu_phi)'*InvOmega_phi*(vecPhi-mu_phi);
                end
                LogPost(i)=-((T*p)/2)*log(2*pi)-(T/2)*log(det(Sigma))-(1/2)*trace(invSigma*(Yphi-Xphi*Pi)'*(Yphi-Xphi*Pi))+LogPrior;
            else
                if ML>0
                    LogPrior=0;
                else
                    LogPrior=-(1/2)*log(det(2*pi*Omega))-(1/2)*(vecPi-mu)'*invOmega*(vecPi-mu);
                end
                LogPost(i)=-((T*p)/2)*log(2*pi)-(T/2)*log(det(Sigma))-(1/2)*trace(invSigma*(Y-X*Pi)'*(Y-X*Pi))+LogPrior;
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
end % End Gibbs sampling

% Computing Mean and Stds of Upsilon
MeanUpsilon=UpsilonCum/(niter-NBurnin);
StdUpsilon=sqrt(UpsilonCumSquare/(niter-NBurnin)-MeanUpsilon.^2);
MeanUpsilon=MeanUpsilon';
StdUpsilon=StdUpsilon';
close(WaitBarHandle)