function []=varfitandresidualsummary(Fitted,Residual,Median_est_fit,Median_est_residual,X,D,K,variables,varlabels,actual_fitted,crossplot,actualres,autocorrres,nlags,histores,EstMedian,IncludeConstant,Dates)
      

[T,p]=size(X);
nvars=length(variables);

if nvars==0
    %nvars=p;
    %variables=1:p;
    return
end

if size(varlabels,1)==0
    for i=1:nvars
        varlabels{i}=strcat('Var',int2str(i));
    end
end

% Transform data to multivariate regression form: Y=XB+E.
if IncludeConstant>0
    D=[D ones(T,1)];
end
[Y,X,D]=transform_to_multireg(X,D,K);
[T,p]=size(Y);
q=size(D,2);
niter=size(Fitted,3);

if EstMedian==1 % EstMedian==1 computes fit based on Posterior median estimates of parameters. Otherwise plots median of estimated fitted values.
    Post_median_fit=Median_est_fit;
    Post_median_residual=Median_est_residual;
else
    Post_median_fit=median(Fitted,3);
    Post_median_residual=median(Residual,3);
end
    
[Nrows,Ncols]=ConstructOptimalSubplot(nvars);

% Plot actual versus fitted
if actual_fitted==1
    FigHandle=figure;
    set(FigHandle,'name','Time Plot of Actual vs Fitted')
    k=0;
    for i=variables
        k=k+1;
        subplot(Nrows,Ncols,k)
        plot(Dates,Y(:,i),'r','LineWidth',1.5)
        hold on
        plot(Dates,Post_median_fit(:,i),'b');
        title(varlabels{i})
        box off
        a=axis;
        axis([Dates(1) Dates(end) a(3:4)]);
        
        if i==1
            legend('Actual','Fit')
        end
    end
end

% Crossplot of actuals vs fitted
if crossplot==1
    FigHandle=figure;
    set(FigHandle,'name','Cross Plot of Actual vs Fitted')
    k=0;
    for i=variables
        k=k+1;
        subplot(Nrows,Ncols,k)
        plot(Y(:,i),Post_median_fit(:,i),'.k');
        title(varlabels{i})
        box off
    end
end

% Plotting actual residuals across time
if actualres==1
    FigHandle=figure;
    set(FigHandle,'name','Time Plots of Residuals')
    k=0;
    for i=variables
        k=k+1;
        subplot(Nrows,Ncols,k)
        plot(Dates,Post_median_residual(:,i),'k','LineWidth',1.5);
        title(varlabels{i})
        box off
        a=axis;
        axis([Dates(1) Dates(end) a(3:4)]);
    end
end

% Computing autocorrelation
if autocorrres==1
    autocorr=zeros(niter,nlags,nvars);
    autocorrsummary=zeros(nlags,3,nvars);
    for var=variables
        for i=1:niter
            autocorr(i,:,var)=sacf(Residual(:,var,i),nlags,1)';
        end
        autocorrsummary(:,2,var)=median(autocorr(:,:,var))';
        s=sort(autocorr(:,:,var));
        if niter>25
            autocorrsummary(:,1,var)=s(round(0.05*niter),:)';
            autocorrsummary(:,3,var)=s(round(0.95*niter),:)';
        end
    end
end

% Residual autocorrelation graphs
if autocorrres==1
    FigHandle=figure;
    set(FigHandle,'name','Residual Autocorrelation')
    k=0;
    for i=variables
        k=k+1;
        subplot(Nrows,Ncols,k)
        h=line([0 nlags],[0 0]);
        set(h,'color','r')
        set(h,'linestyle',':')
        hold on
        if niter>25
            errorbar(1:nlags,autocorrsummary(:,2,i),autocorrsummary(:,1,i),autocorrsummary(:,3,i),'.')
        else
            bar(1:nlags,autocorrsummary(:,2,i))
        end
        title(varlabels{i})
        box off
        a=axis;
        axis([0 nlags a(3:4)]);
    end
end

% Residual histogram
if histores==1
    FigHandle=figure;
    set(FigHandle,'name','Residual Histogram')
    if niter<10
        NHistBins=10;
    else
        NHistBins=sqrt(niter);
    end
    k=0;
    for i=variables
        k=k+1;
        subplot(Nrows,Ncols,k)
        hist(Post_median_residual(:,i),NHistBins);
        title(varlabels{i})
        box off
    end
end
